/*
 * This file is part of Discord4J.
 *
 * Discord4J is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Discord4J is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Discord4J. If not, see <http://www.gnu.org/licenses/>.
 */

package discord4j.rest.request;

import reactor.core.publisher.Mono;

/**
 * Represents a connector executing {@link discord4j.rest.request.DiscordRequest} objects against the Discord REST API.
 */
public interface Router {

    /**
     * Queues a request for execution.
     *
     * @param request the request to queue.
     * @param <T> the request's response type.
     * @return a mono that receives signals based on the request's response.
     */
    <T> Mono<T> exchange(DiscordRequest<T> request);
}
